﻿//Program.cs - Demo3
//F.R.Hubbell 07/06/2010
//Title: Interactive Koch Curve, Left Mouse Button Increases Level, Right Mouse Button Decreases Level
//Platform: C# WPF Visual Studio 2010
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace Demo3
{
    class MainWindow : Window
    {
        Canvas m_canvas;
        Polygon m_polygon;
        double m_size;
        double m_level;
        double m_x;
        double m_y;
        double m_angle;
        double m_aspect;

        [STAThread]
        static void Main(string[] args)
        {
            Application app = new Application();
            app.Run(new MainWindow());
        }

        public MainWindow()
        {
            Title = "Koch Curve - Click On Window - Interation: " + (m_level + 1).ToString();
            WindowState = WindowState.Maximized;
            m_canvas = new Canvas();
            Content = m_canvas;

            m_level = 0;
            m_polygon = new Polygon();
            m_polygon.Stroke = Brushes.Black;
            m_polygon.Points = new PointCollection();

            Show(); //Displaying window is necessary to give values to Actualheight and ActualWidth
            m_size = m_canvas.ActualHeight * 0.80; //Dynamically size to current window
            double x0 = (int)(m_canvas.ActualWidth / 2); ; //Determine center coordinates
            double y0 = (int)(m_canvas.ActualHeight / 2); ;
            m_aspect = 1.0;
            m_x = x0 + 0.5 * m_size; //Compute starting point
            m_y = y0 - 0.3 * m_size;

            snowflake(m_size, m_level);
            m_canvas.Children.Insert(0, m_polygon);
        }

        private void snowflake(double size, double level)
        {
            if (level >= 0)
            {
                m_angle = 0.0;
                for (int i = 0; i < 3; i++)
                {
                    right(120.0);
                    side(size, level);
                }
            }
        }

        private void right(double delta)
        {
            m_angle -= delta;
        }

        private void left(double delta)
        {
            m_angle += delta;
        }

        private void side(double size, double level)
        {
            if (level <= 0 || size <= 1)
            {
                forward(size);
            }
            else
            {
                side(size / 3.0, level - 1);
                left(60.0);
                side(size / 3.0, level - 1);
                right(120.0);
                side(size / 3.0, level - 1);
                left(60.0);
                side(size / 3.0, level - 1);
            }
        }

        private void forward(double amount)
        {
            double x1 = Math.Cos(m_angle * Math.PI / 180.0) * amount * m_aspect + m_x;
            double y1 = Math.Sin(m_angle * Math.PI / 180.0) * amount * -1.0 + m_y;
            m_polygon.Points.Add(new Point(m_x, m_y));
            m_polygon.Points.Add(new Point(x1, y1));
            m_x = x1;
            m_y = y1;
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);
            m_polygon.Points.Clear();
            m_level++;
            Title = "Koch Curve - Click On Window - Interation: " + (m_level + 1).ToString();
            snowflake(m_size, m_level);
        }

        protected override void OnMouseRightButtonDown(MouseButtonEventArgs e)
        {
            base.OnMouseRightButtonDown(e);
            m_polygon.Points.Clear();
            if (m_level >= 0)
            {
                m_level--;
            }
            Title = "Koch Curve - Click On Window - Interation: " + (m_level + 1).ToString();
            snowflake(m_size, m_level);
        }
    }
}
